/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001 phonohawkput

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya;
import aya.*;
import aya.node.*;
import aya.eval.*;
import java.io.*;
import java.util.*;

public class Aya
{
	public static final String CHARSET = "SJIS";
	
	File ghost_dir;
	Dictionary dic;
	GlobalNamespace global_namespace;
	SystemCall syscall;
	
	public Aya(File ghost_dir)
	{
		this.ghost_dir = ghost_dir;
		this.dic = new Dictionary(this);
		this.global_namespace = new GlobalNamespace();
		this.syscall = new SystemCall(this);
		
		// aya.txtがあれば読む。
		File aya_txt = new File(ghost_dir,"aya.txt");
		if (aya_txt.exists())
		{
			loadAyaConfigFile(aya_txt);
		}
		
		// OnLoadを呼ぶ。暫定。
		try {
			Function onload = dic.getFunction("OnLoad");
			if (onload != null) {
				onload.eval(null);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public GlobalNamespace getGlobalNamespace() {
		return global_namespace;
	}
	
	public Dictionary getDictionary() {
		return dic;
	}
	
	public SystemCall getSystemCall() {
		return syscall;
	}
	
	protected void loadAyaConfigFile(File confFile)
    {
		try{
			BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(confFile),CHARSET));
			while (true)
			{
				String line = br.readLine();
				if (line == null) break; // EOF

				int doubleSlash_pos = line.indexOf("//");
				if (doubleSlash_pos != -1)
				{
					line = line.substring(0,doubleSlash_pos);
				}

				if (line.length() == 0) continue;

				int comma_pos = line.indexOf(',');
				if (comma_pos == -1) continue;
				String key = line.substring(0,comma_pos).trim();
				String value = line.substring(comma_pos+1).trim();

				evaluateAyaConfDefinition(key,value);
			}

			br.close();
		}catch (Exception e){
			System.err.println("SCAya: exception occurerd in loading Aya Config File (aya.txt).");
			e.printStackTrace();
		}
    }
    protected void evaluateAyaConfDefinition(String key,String value)
    {
		if (key.equals("dic"))
		{
			// 辞書を読み込む
			File dicfile = new File(ghost_dir,convertWinPathToUnixOne(value));
			dic.loadFile(dicfile);
		}
    }
    public static String convertWinPathToUnixOne(String path)
    {
		// Windowsのパス区切り￥を、Unixのパス区切り/に置き換えます。
		StringBuffer buf = new StringBuffer(path);
		while (true)
		{
			int en_pos = buf.toString().indexOf('\\');
			if (en_pos == -1) en_pos = buf.toString().indexOf('\u00a5');
			if (en_pos == -1) break; // 終了

			buf.setCharAt(en_pos,'/');
		}
		return buf.toString();
    }
	
	public static boolean checkIfUsable(File ghost_dir)
    {
		// 手抜き
		File aya_txt = new File(ghost_dir,"aya.txt");
		if (aya_txt.exists()) return true;
		File aya_dll = new File(ghost_dir,"aya.dll");
		if (aya_dll.exists()) return true;
		return false;
    }
	
	public void terminate()
    {
		//global_namespace.saveToFile();
    }
	
	public static String getModuleName()
    {
		return "Aya";
    }
	
	public String request(String req)
    {
		BufferedReader br = new BufferedReader(new StringReader(req));

		// まずはヘッダを読む
		String header = null;
		try{ header = br.readLine(); }catch (Exception e){}

		// リクエスト全体を読む
		Hashtable hash = new Hashtable();
		String line = null;
		while (true)
		{
			try{ line = br.readLine(); }catch (Exception e){}
			if (line == null) break;
			if (line.indexOf(':') == -1) continue;

			String label = line.substring(0,line.indexOf(':'));
			String data = line.substring(line.indexOf(':')+2); // コロンの次のスペースは無効

			hash.put(label,data);
		}

		return "SHIORI/3.0 204 No Content\r\n\r\n";
	}
}
