/*
 "NiseRingo" is a clone of "Niseharuna" works on MacOS X.
 Copyright (C) 2001-2003 phonohawk

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package aya.node;
import aya.*;
import aya.node.*;
import aya.eval.*;
import java.util.*;

public class Expression
{
	protected static Expression _factory = null;
	
	public static Expression getFactory() {
		if (_factory == null) {
			_factory = new Expression();
		}
		return _factory;
	}

	Aya aya;
	Vector terms; // [Term,Term,...]
	
	protected Expression() {
		// constructor of factory
		this.aya = null;
		this.terms = null;
	}
	
	public Expression(Aya aya) {
		this.aya = aya;
		this.terms = new Vector();
	}
	
	public Expression addTerm(Term t) {
		terms.add(t);
		return this;
	}
	
	public int size() {
		return terms.size();
	}
	
	public VariableReference isVariableOnly() {
		// ̎YȂP̕ϐQƂ݂̂ō\Ă΁AVariableReferenceԂB
		// łȂnullԂB
		
		// ΃AEgB
		if (terms.size() != 1) {
			return null;
		}
		
		Term term = (Term)terms.elementAt(0);
		
		// ̕łȂ΃AEgB
		if (term.getSignal() != 1) {
			return null;
		}
		
		// qłȂ΃AEgB
		if (term.size() != 1) {
			return null;
		}
		
		Factor factor = term.factorAt(0);
		
		// q̕łȂ΃AEgB
		if (factor.getSignal() != 1) {
			return null;
		}
		
		// qVariableFactorȂ炱ԂB
		// FunctionCallFactoȑꍇ́AϐQƂփ_CNg\ȏꍇ݂̂ԂB
		if (factor instanceof VariableFactor) {
			return ((VariableFactor)factor).getVariableReference();
		}
		else if (factor instanceof FunctionCallFactor) {
			FunctionCall fcall = ((FunctionCallFactor)factor).getFunctionCall();
			if (fcall.getArgs() == null) {
				Function f = aya.getDictionary().getFunction(fcall.getName());
				if (f == null) {
					return new VariableReference(fcall.getName());
				}
			}
		}
		
		return null;
	}
	
	public Value eval(Namespace ns) throws Exception {
		Value result = null;
		for (int i = 0; i < terms.size(); i++) {
			Term t = (Term)terms.elementAt(i);
			Value val = t.eval(ns);
			
			if (result == null) {
				result = val;
			}
			else {
				// 񓯎m̉ZA܂͐lm̉Z͉\B
				if (result.isString() && val.isString()) {
					result.setString(result.getString() + val.getString());
				}
				else if (result.isNumeric() && val.isNumeric()) {
					// ǂ炩AʂB
					if (result.isReal() || val.isReal()) {
						result.setReal(result.getReal() + val.getReal());
					}
					else {
						result.setInteger(result.getInteger() + val.getInteger());
					}
				}
				else {
					throw new Exception("Type mismatch: add/sub with string and numeric.");
				}
			}
		}
		return result;
	}
	
	// factory
	public Term newTerm() {
		return new Term();
	}
	
	public Factor newFactor(Expression expr) {
		return new ExpressionFactor(expr);
	}
	
	public Factor newFactor(double num) {
		return new NumericFactor(num);
	}
	
	public Factor newFactor(String str) {
		return new StringFactor(str);
	}
	
	public Factor newFactor(Aya aya,FunctionCall fcall) {
		return new FunctionCallFactor(aya,fcall);
	}
	
	public Factor newFactor(Aya aya,VariableReference vref) {
		return new VariableFactor(aya,vref);
	}
	
	public class Term
	{
		Vector factors; // [Factor,Factor,...]
		Vector combinations; // [Character,Character,...]
		int signal;
		
		public Term() {
			factors = new Vector();
			combinations = new Vector();
			signal = 1;
		}
		
		public Term setSignal(int signal) {
			if (signal == 1 || signal == -1) {
				this.signal = signal;
			}
			else {
				try {
					throw new Exception("Signal must be 1 or -1. "+signal+" is invalid.");
				}
				catch (Exception e) {}
			}
			return this;
		}
		
		public int getSignal() {
			return signal;
		}
		
		public void addFactor(char combination,Factor factor) {
			// ꂪŏ̈qł΁Acombination͖B
			if (factors.size() != 0) {
				combinations.add(new Character(combination));
			}
			factors.add(factor);
		}
		
		public int size() {
			return factors.size();
		}
		
		public Factor factorAt(int i) {
			return (Factor)factors.elementAt(i);
		}
		
		public Value eval(Namespace ns) throws Exception {
			Value result = null;
			for (int i = 0; i < factors.size(); i++) {
				Factor factor = (Factor)factors.elementAt(i);
				
				// Ȃ̂ɕȂG[B
				if (factor instanceof StringFactor && factor.getSignal() != 1) {
					throw new Exception("Signal of string value must be positive.");
				}
				
				if (i == 0) {
					// ŏȂ̂combinationB
					result = factor.eval(ns);
				}
				else {
					// Ȃ̂ɕ̈q΃G[B
					if (result.isString()) {
						throw new Exception("Only operator '+' can be used with strings.");
					}
					
					Value val = factor.eval(ns);
					char comb = ((Character)combinations.elementAt(i-1)).charValue();
					if (comb == '*') {
						// resultval̂ǂ炩ȂAʂB
						if (result.isReal() || val.isReal()) {
							result.setReal(result.getReal() * val.getReal());
						}
						else {
							result.setInteger(result.getInteger() * val.getInteger());
						}
					}
					else if (comb == '/') {
						// ʂɂȂΐBȂȂΎB
						double d = result.getReal() / val.getReal();
						if (d == Math.floor(d)) {
							result.setInteger((long)d);
						}
						else {
							result.setReal(d);
						}
					}
					else if (comb == '%') {
						// ɐB
						result.setInteger(result.getInteger() % val.getInteger());
					}
					else {
						throw new Exception("Unsupported combinational operator '"+comb+"'.");
					}
				}
			}
			
			if (result == null) {
				return null;
			}
			else if (result.isString() && signal == -1) {
				throw new Exception("Signal of string value must be positive.");
			}
			else if (result.isInteger() && signal == -1) {
				result.setInteger(result.getInteger() * -1);
			}
			else if (result.isReal() && signal == -1) {
				result.setReal(result.getReal() * -1);
			}
			return result;
		}
	}
	
	public class Factor
	{
		// q̗vfɂ͎̂̂B
		//  , l萔 , 萔 , ֐ďo , ϐ
		int signal;
		
		public Factor() {
			signal = 1;
		}
		
		public int getSignal() {
			return signal;
		}
		
		public Factor setSignal(int signal) {
			if (signal == 1 || signal == -1) {
				this.signal = signal;
			}
			else {
				try {
					throw new Exception("Signal must be 1 or -1. "+signal+" is invalid.");
				}
				catch (Exception e) {}
			}
			return this;
		}
		
		public Value eval(Namespace ns) throws Exception {
			throw new Exception("Abstract method \"eval\" of Expression.Factor has been called directory.");
		}
	}
	
	public class ExpressionFactor extends Factor
	{
		Expression expr;
		
		public ExpressionFactor(Expression expr) {
			this.expr = expr;
		}
		
		public Value eval(Namespace ns) throws Exception {
			return expr.eval(ns);
		}
	}
	
	public class NumericFactor extends Factor
	{
		double num;
		
		public NumericFactor(double num) {
			this.num = num;
		}
		
		public Value eval(Namespace ns) throws Exception {
			// Ȃ琮^ŕԂB
			if (num == Math.floor(num)) {
				return new Value((long)num);
			}
			else {
				return new Value(num);
			}
		}
	}
	
	public class StringFactor extends Factor
	{
		String str;
		
		public StringFactor(String str) {
			this.str = str;
		}
		
		public Value eval(Namespace ns) throws Exception {
			return new Value(str);
		}
	}
	
	public class FunctionCallFactor extends Factor
	{
		Aya aya;
		FunctionCall fcall;
		
		public FunctionCallFactor(Aya aya,FunctionCall fcall) {
			this.aya = aya;
			this.fcall = fcall;
		}
		
		public FunctionCall getFunctionCall() {
			return fcall;
		}
		
		public Value eval(Namespace ns) throws Exception {
			Function f = aya.getDictionary().getFunction(fcall.getName());
			if (f != null) {
				Vector args = fcall.getArgs();
				Vector args_for_f = null;
				
				if (args != null) {
					args_for_f = new Vector();
					for (int i = 0; i < args.size(); i++) {
						FunctionCall.Argument arg = (FunctionCall.Argument)args.elementAt(i);
						if (arg instanceof FunctionCall.ExpressionArgument) {
							Expression expr = ((FunctionCall.ExpressionArgument)arg).getExpression();
							// ̎P̕ϐQƂłAYꍇ́A
							// Öق̕ϐ|C^쐬B
							VariableReference vref = expr.isVariableOnly();
							if (vref != null && vref.getIndex() == null) {
								args_for_f.add(new VariablePointer(ns,vref,true));
							}
							else {
								args_for_f.add(expr.eval(ns));
							}
						}
						else if (arg instanceof FunctionCall.ReferenceArgument) {
							VariableReference vref = ((FunctionCall.ReferenceArgument)arg).getReference();
							args_for_f.add(new VariablePointer(ns,vref));
						}
						else {
							throw new Exception("Internal Error: an object of "+arg.getClass().getName()+" was in array part of Variable illegally.");
						}
					}
				}
				return f.eval(args_for_f);
			}
			else {
				// ̖`֐̌ďóAϐ̎QƂɃ_CNgB
				if (fcall.getArgs() == null) {
					Namespace space = (fcall.getName().charAt(0) != '_' ? aya.getGlobalNamespace() : ns);
					Variable v = space.get(fcall.getName());
					if (v != null) {
						return v.getValue();
					}
				}
				return new Value(""); // ̓ɂ̗͌]nB
			}
		}
	}
	
	public class VariableFactor extends Factor
	{
		Aya aya;
		VariableReference vref;
		
		public VariableFactor(Aya aya,VariableReference vref) {
			this.aya = aya;
			this.vref = vref;
		}
		
		public VariableReference getVariableReference() {
			return vref;
		}
		
		public Value eval(Namespace ns) throws Exception {
			Namespace space = (vref.isGlobal() ? aya.getGlobalNamespace() : ns);
			Variable v = space.define(vref.getName());
			
			if (vref.getIndex() != null) {
				// ȈՔz̓YB
				return v.getValue(vref.evalIndex(ns));
			}
			else {
				return v.getValue();
			}
		}
	}
}
